@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c 6273 2008-05-09 16:59:28Z pborelli $
+ * $Id: gedit-commands-file.c 6379 2008-08-10 16:40:21Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -510,8 +510,8 @@ open_location_dialog_response_cb (GeditOpenLocationDialog *dlg,
 				  GeditWindow             *window)
 {
 	GFile *location;
-	GSList *uris = NULL;
 	const GeditEncoding *encoding;
+	GSList *uris = NULL;
 
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -525,17 +525,25 @@ open_location_dialog_response_cb (GeditOpenLocationDialog *dlg,
 	location = gedit_open_location_dialog_get_location (dlg);
 	encoding = gedit_open_location_dialog_get_encoding (dlg);
 
-	uris = g_slist_prepend (uris, g_file_get_uri (location));
+	if (location != NULL)
+	{
+		uris = g_slist_prepend (uris, g_file_get_uri (location));
+
+		g_object_unref (location);
+	}
 
 	gtk_widget_destroy (GTK_WIDGET (dlg));
 
-	gedit_commands_load_uris (window,
-				  uris,
-				  encoding,
-				  0);
+	if (uris != NULL)
+	{
+		gedit_commands_load_uris (window,
+					  uris,
+					  encoding,
+					  0);
 
-	g_slist_foreach (uris, (GFunc) g_free, NULL);
-	g_slist_free (uris);
+		g_slist_foreach (uris, (GFunc) g_free, NULL);
+		g_slist_free (uris);
+	}
 }
 
 void
@@ -704,7 +712,7 @@ save_dialog_response_cb (GeditFileChooserDialog *dialog,
 		doc = gedit_tab_get_document (tab);
 		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
-		uri_for_display = gedit_utils_format_uri_for_display (uri);
+		uri_for_display = gedit_utils_uri_for_display (uri);
 
 		gedit_statusbar_flash_message (GEDIT_STATUSBAR (window->priv->statusbar),
 					        window->priv->generic_message_cid,
