@@ -31,10 +31,7 @@
 #endif
 
 #include <glib/gi18n.h>
-
-#include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 
 #include "gedit-app.h"
 #include "gedit-notebook.h"
@@ -42,8 +39,8 @@
 #include "gedit-utils.h"
 #include "gedit-message-area.h"
 #include "gedit-io-error-message-area.h"
-#include "gedit-print.h"
-#include "gedit-print-job-preview.h"
+#include "gedit-print-job.h"
+#include "gedit-print-preview.h"
 #include "gedit-progress-message-area.h"
 #include "gedit-debug.h"
 #include "gedit-prefs-manager-app.h"
@@ -229,15 +226,7 @@ static void
 gedit_tab_finalize (GObject *object)
 {
 	GeditTab *tab = GEDIT_TAB (object);
-	
-	if (tab->priv->print_job != NULL)
-	{
-		gedit_debug_message (DEBUG_TAB, "Cancelling printing");
-		
-		gtk_source_print_job_cancel (GTK_SOURCE_PRINT_JOB (tab->priv->print_job));
-		g_object_unref (tab->priv->print_job);
-	}
-	
+
 	if (tab->priv->timer != NULL)
 		g_timer_destroy (tab->priv->timer);
 
@@ -264,7 +253,9 @@ gedit_tab_class_init (GeditTabClass *klass)
 							      "Name",
 							      "The tab's name",
 							      NULL,
-							      G_PARAM_READABLE));
+							      G_PARAM_READABLE |
+							      G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (object_class,
 					 PROP_STATE,
 					 g_param_spec_enum ("state",
@@ -272,26 +263,29 @@ gedit_tab_class_init (GeditTabClass *klass)
 							    "The tab's state",
 							    GEDIT_TYPE_TAB_STATE,
 							    GEDIT_TAB_STATE_NORMAL,
-							    G_PARAM_READABLE));							      	
-							      
+							    G_PARAM_READABLE |
+							    G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (object_class,
 					 PROP_AUTO_SAVE,
 					 g_param_spec_boolean ("autosave",
 							       "Autosave",
 							       "Autosave feature",
 							       TRUE,
-							       G_PARAM_READWRITE));
+							       G_PARAM_READWRITE |
+							       G_PARAM_STATIC_STRINGS));
 
 	g_object_class_install_property (object_class,
 					 PROP_AUTO_SAVE_INTERVAL,
-					 g_param_spec_int ("autosave_interval",
+					 g_param_spec_int ("autosave-interval",
 							   "AutosaveInterval",
 							   "Time between two autosaves",
 							   0,
 							   G_MAXINT,
 							   0,
-							   G_PARAM_READWRITE));
-							      
+							   G_PARAM_READWRITE |
+							   G_PARAM_STATIC_STRINGS));
+
 	g_type_class_add_private (object_class, sizeof (GeditTabPrivate));
 }
 
@@ -756,7 +750,7 @@ show_saving_message_area (GeditTab *tab)
 
 		from = short_name;
 
-		to = gedit_utils_format_uri_for_display (tab->priv->tmp_save_uri);
+		to = gedit_utils_uri_for_display (tab->priv->tmp_save_uri);
 
 		str = gedit_utils_str_middle_truncate (to, 
 						       MAX (20, MAX_MSG_LENGTH - len));
@@ -798,9 +792,9 @@ show_saving_message_area (GeditTab *tab)
 }
 
 static void
-message_area_set_progress (GeditTab	    *tab,
-			   GnomeVFSFileSize  size,
-			   GnomeVFSFileSize  total_size)
+message_area_set_progress (GeditTab *tab,
+			   goffset   size,
+			   goffset   total_size)
 {
 	if (tab->priv->message_area == NULL)
 		return;
@@ -832,10 +826,10 @@ message_area_set_progress (GeditTab	    *tab,
 }
 
 static void
-document_loading (GeditDocument    *document,
-		  GnomeVFSFileSize  size,
-		  GnomeVFSFileSize  total_size,
-		  GeditTab         *tab)
+document_loading (GeditDocument *document,
+		  goffset        size,
+		  goffset        total_size,
+		  GeditTab      *tab)
 {
 	double et;
 
@@ -901,6 +895,7 @@ document_loaded (GeditDocument *document,
 		 GeditTab      *tab)
 {
 	GtkWidget *emsg;
+	GFile *location;
 	gchar *uri;
 	const GeditEncoding *encoding;
 
@@ -917,6 +912,7 @@ document_loaded (GeditDocument *document,
 
 	set_message_area (tab, NULL);
 
+	location = gedit_document_get_location (document);
 	uri = gedit_document_get_uri (document);
 
 	if (error != NULL)
@@ -930,7 +926,7 @@ document_loaded (GeditDocument *document,
 
 		if (error->domain == GEDIT_DOCUMENT_ERROR)
 		{
-			if (error->code == GNOME_VFS_ERROR_CANCELLED)
+			if (error->code == G_IO_ERROR_CANCELLED)
 			{
 				/* remove the tab, but in an idle handler, since
 				 * we are in the handler of doc loaded and we 
@@ -973,7 +969,7 @@ document_loaded (GeditDocument *document,
 		{
 			g_return_if_fail ((error->domain == G_CONVERT_ERROR) ||
 			      		  (error->domain == GEDIT_CONVERT_ERROR));
-			
+
 			// TODO: different error messages if tab->priv->state == GEDIT_TAB_STATE_REVERTING?
 			// note that while reverting encoding should be ok, so this is unlikely to happen
 			emsg = gedit_conversion_error_while_loading_message_area_new (
@@ -993,6 +989,8 @@ document_loaded (GeditDocument *document,
 							 GTK_RESPONSE_CANCEL);
 
 		gtk_widget_show (emsg);
+
+		g_object_unref (location);
 		g_free (uri);
 
 		return;
@@ -1019,12 +1017,12 @@ document_loaded (GeditDocument *document,
 			
 			if (d != document)
 			{
-				gchar *u;
+				GFile *loc;
 
-				u = gedit_document_get_uri (d);
+				loc = gedit_document_get_location (d);
 
-				if ((u != NULL) &&
-			    	    gnome_vfs_uris_match (uri, u))
+				if ((loc != NULL) &&
+			    	    g_file_equal (location, loc))
 			    	{
 			    		GtkWidget *w;
 			    		GeditView *view;
@@ -1047,12 +1045,11 @@ document_loaded (GeditDocument *document,
 							  G_CALLBACK (file_already_open_warning_message_area_response),
 							  tab);
 
-					g_free (u);
-
+			    		g_object_unref (loc);
 			    		break;
 			    	}
 
-				g_free (u);
+				g_object_unref (loc);
 			}
 		}
 
@@ -1066,6 +1063,7 @@ document_loaded (GeditDocument *document,
 	}
 
  end:
+	g_object_unref (location);
 	g_free (uri);
 
 	tab->priv->tmp_line_pos = 0;
@@ -1074,8 +1072,8 @@ document_loaded (GeditDocument *document,
 
 static void
 document_saving (GeditDocument    *document,
-		 GnomeVFSFileSize  size,
-		 GnomeVFSFileSize  total_size,
+		 goffset  size,
+		 goffset  total_size,
 		 GeditTab         *tab)
 {
 	double et;
@@ -1485,6 +1483,17 @@ view_focused_in (GtkWidget     *widget,
 	return FALSE;
 }
 
+static GMountOperation *
+tab_mount_operation_factory (GeditDocument *doc,
+			     gpointer userdata)
+{
+	GeditTab *tab = GEDIT_TAB (userdata);
+	GtkWidget *window;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (tab));
+	return gtk_mount_operation_new (GTK_WINDOW (window));
+}
+
 static void
 gedit_tab_init (GeditTab *tab)
 {
@@ -1524,6 +1533,10 @@ gedit_tab_init (GeditTab *tab)
 	doc = gedit_document_new ();
 	g_object_set_data (G_OBJECT (doc), GEDIT_TAB_KEY, tab);
 
+	_gedit_document_set_mount_operation_factory (doc,
+						     tab_mount_operation_factory,
+						     tab);
+
 	tab->priv->view = gedit_view_new (doc);
 	g_object_unref (doc);
 	gtk_widget_show (tab->priv->view);
@@ -1667,11 +1680,6 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 	gchar *uri;
 	gchar *ruri;
 	gchar *ruri_markup;
-	gchar *mime_type;
-	const gchar *mime_description = NULL;
-	gchar *mime_full_description; 
-	gchar *encoding;
-	const GeditEncoding *enc;
 
 	g_return_val_if_fail (GEDIT_IS_TAB (tab), NULL);
 
@@ -1687,6 +1695,12 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 
 	switch (tab->priv->state)
 	{
+		gchar *mime_type;
+		gchar *mime_description;
+		gchar *mime_full_description; 
+		gchar *encoding;
+		const GeditEncoding *enc;
+
 		case GEDIT_TAB_STATE_LOADING_ERROR:
 			tip = g_strdup_printf (_("Error opening file %s"),
 					       ruri_markup);
@@ -1703,7 +1717,7 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 			break;			
 		default:
 			mime_type = gedit_document_get_mime_type (doc);
-			mime_description = gnome_vfs_mime_get_description (mime_type);
+			mime_description = g_content_type_get_description (mime_type);
 
 			if (mime_description == NULL)
 				mime_full_description = g_strdup (mime_type);
@@ -1711,6 +1725,7 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 				mime_full_description = g_strdup_printf ("%s (%s)", 
 						mime_description, mime_type);
 
+			g_free (mime_description);
 			g_free (mime_type);
 
 			enc = gedit_document_get_encoding (doc);
@@ -1729,7 +1744,7 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 
 			g_free (encoding);
 			g_free (mime_full_description);
-			
+
 			break;
 	}
 	
@@ -1771,33 +1786,10 @@ resize_icon (GdkPixbuf *pixbuf,
 		g_object_unref (pixbuf);
 		pixbuf = scaled_pixbuf;
 	}
-	
+
 	return pixbuf;
 }
 
-static GdkPixbuf *
-get_icon (GtkIconTheme *theme, 
-	  const gchar  *uri,
-	  const gchar  *mime_type, 
-	  gint          size)
-{
-	gchar *icon;
-	GdkPixbuf *pixbuf;
-	
-	icon = gnome_icon_lookup (theme, NULL, uri, NULL, NULL,
-				  mime_type, 0, NULL);
-	
-
-	g_return_val_if_fail (icon != NULL, NULL);
-
-	pixbuf = gtk_icon_theme_load_icon (theme, icon, size, 0, NULL);
-	g_free (icon);
-	if (pixbuf == NULL)
-		return NULL;
-		
-	return resize_icon (pixbuf, size);
-}
-
 static GdkPixbuf *
 get_stock_icon (GtkIconTheme *theme, 
 		const gchar  *stock,
@@ -1812,6 +1804,51 @@ get_stock_icon (GtkIconTheme *theme,
 	return resize_icon (pixbuf, size);
 }
 
+static GdkPixbuf *
+get_icon (GtkIconTheme *theme, 
+	  GFile        *location,
+	  gint          size)
+{
+	GdkPixbuf *pixbuf;
+	GtkIconInfo *icon_info;
+	GFileInfo *info;
+	GIcon *gicon;
+
+	if (location == NULL)
+		return get_stock_icon (theme, GTK_STOCK_FILE, size);
+
+	/* FIXME: Doing a sync stat is bad, this should be fixed */
+	info = g_file_query_info (location, 
+	                          G_FILE_ATTRIBUTE_STANDARD_ICON, 
+	                          G_FILE_QUERY_INFO_NONE, 
+	                          NULL, 
+	                          NULL);
+	if (info == NULL)
+		return get_stock_icon (theme, GTK_STOCK_FILE, size);
+
+	gicon = g_file_info_get_icon (info);
+
+	if (gicon == NULL)
+	{
+		g_object_unref (info);
+		return get_stock_icon (theme, GTK_STOCK_FILE, size);
+	}
+
+	icon_info = gtk_icon_theme_lookup_by_gicon (theme, gicon, size, 0);
+	g_object_unref (info);	
+	
+	if (icon_info == NULL)
+		return get_stock_icon (theme, GTK_STOCK_FILE, size);
+	
+	pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+	gtk_icon_info_free (icon_info);
+	
+	if (pixbuf == NULL)
+		return get_stock_icon (theme, GTK_STOCK_FILE, size);
+		
+	return resize_icon (pixbuf, size);
+}
+
 /* FIXME: add support for theme changed. I think it should be as easy as
    call g_object_notify (tab, "name") when the icon theme changes */
 GdkPixbuf *
@@ -1884,19 +1921,16 @@ _gedit_tab_get_icon (GeditTab *tab)
 
 		default:
 		{
-			gchar *raw_uri;
-			gchar *mime_type;
+			GFile *location;
 			GeditDocument *doc;
 
 			doc = gedit_tab_get_document (tab);
 
-			raw_uri = gedit_document_get_uri (doc);
-			mime_type = gedit_document_get_mime_type (doc);
+			location = gedit_document_get_location (doc);
+			pixbuf = get_icon (theme, location, icon_size);
 
-			pixbuf = get_icon (theme, raw_uri, mime_type, icon_size);
-
-			g_free (raw_uri);
-			g_free (mime_type);
+			if (location)
+				g_object_unref (location);
 		}
 	}
 
@@ -2144,6 +2178,135 @@ _gedit_tab_save_as (GeditTab            *tab,
 	gedit_document_save_as (doc, uri, encoding, tab->priv->save_flags);
 }
 
+#define GEDIT_PAGE_SETUP_KEY "gedit-page-setup-key"
+#define GEDIT_PRINT_SETTINGS_KEY "gedit-print-settings-key"
+
+static GtkPageSetup *
+get_page_setup (GeditTab *tab)
+{
+	gpointer data;
+	GeditDocument *doc;
+
+	doc = gedit_tab_get_document (tab);
+
+	data = g_object_get_data (G_OBJECT (doc),
+				  GEDIT_PAGE_SETUP_KEY);
+
+	if (data == NULL)
+	{
+		return _gedit_app_get_default_page_setup (gedit_app_get_default());
+	}
+	else
+	{
+		return gtk_page_setup_copy (GTK_PAGE_SETUP (data));
+	}
+}
+
+static GtkPrintSettings *
+get_print_settings (GeditTab *tab)
+{
+	gpointer data;
+	GeditDocument *doc;
+
+	doc = gedit_tab_get_document (tab);
+
+	data = g_object_get_data (G_OBJECT (doc),
+				  GEDIT_PRINT_SETTINGS_KEY);
+
+	if (data == NULL)
+	{
+		return _gedit_app_get_default_print_settings (gedit_app_get_default());
+	}
+	else
+	{
+		return gtk_print_settings_copy (GTK_PRINT_SETTINGS (data));
+	}
+}
+
+/* FIXME: show the message area only if the operation will be "long" */
+static void
+printing_cb (GeditPrintJob       *job,
+	     GeditPrintJobStatus  status,
+	     GeditTab            *tab)
+{
+	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));	
+
+	gtk_widget_show (tab->priv->message_area);
+
+	gedit_progress_message_area_set_text (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+					      gedit_print_job_get_status_string (job));
+
+	gedit_progress_message_area_set_fraction (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+						  gedit_print_job_get_progress (job));
+}
+
+static void
+done_printing_cb (GeditPrintJob       *job,
+		  GeditPrintJobResult  result,
+		  const GError        *error,
+		  GeditTab            *tab)
+{
+	GeditView *view;
+
+	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINT_PREVIEWING ||
+			  tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW ||
+			  tab->priv->state == GEDIT_TAB_STATE_PRINTING);
+
+	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
+		/* print preview has been destroyed... */
+		tab->priv->print_preview = NULL;
+	}
+	else
+	{
+		g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+
+		set_message_area (tab, NULL); /* destroy the message area */
+	}
+
+	// TODO: check status and error
+
+	/* Save the print settings */ 
+	if (result ==  GEDIT_PRINT_JOB_RESULT_OK)
+	{
+		GeditDocument *doc;
+		GtkPrintSettings *settings;
+
+		doc = gedit_tab_get_document (tab);
+
+		settings = gedit_print_job_get_print_settings (job);
+
+		/* remember them for this document */
+		g_object_set_data_full (G_OBJECT (doc),
+					GEDIT_PRINT_SETTINGS_KEY,
+					g_object_ref (settings),
+					(GDestroyNotify)g_object_unref);
+
+		/* make them the default */
+		_gedit_app_set_default_print_settings (gedit_app_get_default (),
+						       settings);
+	}
+
+#if 0
+	if (tab->priv->print_preview != NULL)
+	{
+		/* If we were printing while showing the print preview,
+		   see bug #352658 */
+		gtk_widget_destroy (tab->priv->print_preview);
+		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
+	}
+#endif
+
+	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+
+	view = gedit_tab_get_view (tab);
+	gtk_widget_grab_focus (GTK_WIDGET (view));
+
+ 	g_object_unref (tab->priv->print_job);
+	tab->priv->print_job = NULL;
+}
+
+#if 0
 static void
 print_preview_destroyed (GtkWidget *preview,
 			 GeditTab  *tab)
@@ -2169,9 +2332,41 @@ print_preview_destroyed (GtkWidget *preview,
 		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
 	}	
 }
+#endif
 
 static void
-set_print_preview (GeditTab  *tab, GtkWidget *print_preview)
+show_preview_cb (GeditPrintJob       *job,
+		 GeditPrintPreview   *preview,
+		 GeditTab            *tab)
+{
+//	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINT_PREVIEWING);
+	g_return_if_fail (tab->priv->print_preview == NULL);
+
+	set_message_area (tab, NULL); /* destroy the message area */
+
+	tab->priv->print_preview = GTK_WIDGET (preview);
+	gtk_box_pack_end (GTK_BOX (tab),
+			  tab->priv->print_preview,
+			  TRUE,
+			  TRUE,
+			  0);
+	gtk_widget_show (tab->priv->print_preview);
+	gtk_widget_grab_focus (tab->priv->print_preview);
+
+/* when the preview gets destroyed we get "done" signal
+	g_signal_connect (tab->priv->print_preview,
+			  "destroy",
+			  G_CALLBACK (print_preview_destroyed),
+			  tab);	
+*/
+	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
+}
+
+#if 0
+
+static void
+set_print_preview (GeditTab  *tab,
+		   GtkWidget *print_preview)
 {
 	if (tab->priv->print_preview == print_preview)
 		return;
@@ -2195,38 +2390,6 @@ set_print_preview (GeditTab  *tab, GtkWidget *print_preview)
 			  tab);
 }
 
-#define MIN_PAGES 15
-
-static void
-print_page_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
-{
-	gchar *str;
-	gint page_num;
-	gint total;
-
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));	
-
-	total = gtk_source_print_job_get_page_count (pjob);
-	
-	if (total < MIN_PAGES)
-		return;
-
-	page_num = gtk_source_print_job_get_page (pjob);
-			
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));	
-	
-	gtk_widget_show (tab->priv->message_area);
-	
-	str = g_strdup_printf (_("Rendering page %d of %d..."), page_num, total);
-
-	gedit_progress_message_area_set_text (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
-					      str);
-	g_free (str);
-	
-	gedit_progress_message_area_set_fraction (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
-						  1.0 * page_num / total);
-}
-
 static void
 preview_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 {
@@ -2249,73 +2412,26 @@ preview_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
 }
 
-static void
-print_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
-{
-	GnomePrintJob *gjob;
-	GeditView *view;
 
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
-
-	set_message_area (tab, NULL); /* destroy the message area */
-
-	gjob = gtk_source_print_job_get_print_job (pjob);
-
-	gnome_print_job_print (gjob);
- 	g_object_unref (gjob);
-
-	gedit_print_job_save_config (GEDIT_PRINT_JOB (pjob));
-
-	g_object_unref (pjob);
-
-	if (tab->priv->print_preview != NULL)
-	{
-		/* If we were printing while showing the print preview,
-		   see bug #352658 */
-		gtk_widget_destroy (tab->priv->print_preview);
-		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
-	}
-
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-
-	view = gedit_tab_get_view (tab);
-	gtk_widget_grab_focus (GTK_WIDGET (view));
-}
+#endif
 
 static void
 print_cancelled (GeditMessageArea *area,
                  gint              response_id,
                  GeditTab         *tab)
 {
-	GeditView *view;
-
 	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
-	
-	gtk_source_print_job_cancel (GTK_SOURCE_PRINT_JOB (tab->priv->print_job));
-	g_object_unref (tab->priv->print_job);
 
-	set_message_area (tab, NULL); /* destroy the message area */
+	gedit_print_job_cancel (tab->priv->print_job);
 
-	if (tab->priv->print_preview != NULL)
-	{
-		/* If we were printing while showing the print preview,
-		   see bug #352658 */        	
-		gtk_widget_destroy (tab->priv->print_preview);
-		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
-	}
-
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-
-	view = gedit_tab_get_view (tab);
-	gtk_widget_grab_focus (GTK_WIDGET (view));
+	g_debug ("print_cancelled");
 }
 
 static void
-show_printing_message_area (GeditTab      *tab,
-			    gboolean       preview)
+show_printing_message_area (GeditTab *tab, gboolean preview)
 {
 	GtkWidget *area;
-	
+
 	if (preview)
 		area = gedit_progress_message_area_new (GTK_STOCK_PRINT_PREVIEW,
 							"",
@@ -2324,7 +2440,6 @@ show_printing_message_area (GeditTab      *tab,
 		area = gedit_progress_message_area_new (GTK_STOCK_PRINT,
 							"",
 							TRUE);
-	
 
 	g_signal_connect (area,
 			  "response",
@@ -2334,100 +2449,137 @@ show_printing_message_area (GeditTab      *tab,
 	set_message_area (tab, area);
 }
 
-void 
-_gedit_tab_print (GeditTab      *tab,
-		  gpointer       print_job,
-		  GtkTextIter   *start, 
-		  GtkTextIter   *end)
+static void
+page_setup_done_cb (GtkPageSetup *setup,
+		    GeditTab     *tab)
 {
-	GeditPrintJob *pjob;
-	GeditDocument *doc;
-
-	pjob = (GeditPrintJob *) print_job;
-
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
-	g_return_if_fail (start != NULL);
-	g_return_if_fail (end != NULL);	
-		
-	doc = GEDIT_DOCUMENT (gtk_source_print_job_get_buffer (GTK_SOURCE_PRINT_JOB (pjob)));
-	g_return_if_fail (doc != NULL);
-	g_return_if_fail (gedit_tab_get_document (tab) == doc);
-	g_return_if_fail (gtk_text_iter_get_buffer (start) == GTK_TEXT_BUFFER (doc));
-	g_return_if_fail (gtk_text_iter_get_buffer (end) == GTK_TEXT_BUFFER (doc));	
-	
-	g_return_if_fail (tab->priv->print_job == NULL);
-	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_NORMAL) ||
-			  (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW));
-	
-	g_object_ref (pjob);
-	tab->priv->print_job = pjob;
-	g_object_add_weak_pointer (G_OBJECT (pjob), 
-				   (gpointer *) &tab->priv->print_job);
-	
-	show_printing_message_area (tab, FALSE);
-
-	g_signal_connect (pjob, "begin_page", (GCallback) print_page_cb, tab);
-	g_signal_connect (pjob, "finished", (GCallback) print_finished_cb, tab);
-
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINTING);
-	
-	if (!gtk_source_print_job_print_range_async (GTK_SOURCE_PRINT_JOB (pjob), start, end))
+	if (setup != NULL)
 	{
-		/* FIXME: go in error state */
-		gtk_text_view_set_editable (GTK_TEXT_VIEW (tab->priv->view), 
-					    !tab->priv->not_editable);
-		g_warning ("Async print preview failed");
-		g_object_unref (pjob);
+		GeditDocument *doc;
+
+		doc = gedit_tab_get_document (tab);
+
+		/* remember it for this document */
+		g_object_set_data_full (G_OBJECT (doc),
+					GEDIT_PAGE_SETUP_KEY,
+					g_object_ref (setup),
+					(GDestroyNotify)g_object_unref);
+
+		/* make it the default */
+		_gedit_app_set_default_page_setup (gedit_app_get_default(),
+						   setup);
 	}
 }
 
-void
-_gedit_tab_print_preview (GeditTab      *tab,
-			  gpointer       print_job,
-			  GtkTextIter   *start, 
-			  GtkTextIter   *end)
+void 
+_gedit_tab_page_setup (GeditTab *tab)
 {
-	GeditPrintJob *pjob;
-	GeditDocument *doc;
-
-	pjob = (GeditPrintJob *) print_job;
+	GtkPageSetup *setup;
+	GtkPrintSettings *settings;
 
 	g_return_if_fail (GEDIT_IS_TAB (tab));
-	g_return_if_fail (GEDIT_IS_PRINT_JOB (pjob));
-	g_return_if_fail (start != NULL);
-	g_return_if_fail (end != NULL);	
-		
-	doc = GEDIT_DOCUMENT (gtk_source_print_job_get_buffer (GTK_SOURCE_PRINT_JOB (pjob)));
-	g_return_if_fail (doc != NULL);
-	g_return_if_fail (gedit_tab_get_document (tab) == doc);
-	g_return_if_fail (gtk_text_iter_get_buffer (start) == GTK_TEXT_BUFFER (doc));
-	g_return_if_fail (gtk_text_iter_get_buffer (end) == GTK_TEXT_BUFFER (doc));	
-	
+
+	setup = get_page_setup (tab);
+	settings = get_print_settings (tab);
+
+	gtk_print_run_page_setup_dialog_async (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))),
+		 			       setup,
+		 			       settings,
+					       (GtkPageSetupDoneFunc) page_setup_done_cb,
+					       tab);
+
+	/* CHECK: should we unref setup and settings? */
+}
+
+static void
+gedit_tab_print_or_print_preview (GeditTab                *tab,
+				  GtkPrintOperationAction  print_action)
+{
+	GeditView *view;
+	gboolean is_preview;
+	GtkPageSetup *setup;
+	GtkPrintSettings *settings;
+	GtkPrintOperationResult res;
+	GError *error = NULL;
+
 	g_return_if_fail (tab->priv->print_job == NULL);
 	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_NORMAL);
-	
-	g_object_ref (pjob);
-	tab->priv->print_job = pjob;
-	g_object_add_weak_pointer (G_OBJECT (pjob), 
+
+	view = gedit_tab_get_view (tab);
+
+	is_preview = (print_action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
+
+	tab->priv->print_job = gedit_print_job_new (view);
+	g_object_add_weak_pointer (G_OBJECT (tab->priv->print_job), 
 				   (gpointer *) &tab->priv->print_job);
-	
-	show_printing_message_area (tab, TRUE);
 
-	g_signal_connect (pjob, "begin_page", (GCallback) print_page_cb, tab);
-	g_signal_connect (pjob, "finished", (GCallback) preview_finished_cb, tab);
+	show_printing_message_area (tab, is_preview);
 
-	gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINT_PREVIEWING);
-	
-	if (!gtk_source_print_job_print_range_async (GTK_SOURCE_PRINT_JOB (pjob), start, end))
+	g_signal_connect (tab->priv->print_job,
+			  "printing",
+			  G_CALLBACK (printing_cb),
+			  tab);
+	g_signal_connect (tab->priv->print_job,
+			  "show-preview",
+			  G_CALLBACK (show_preview_cb),
+			  tab);
+	g_signal_connect (tab->priv->print_job,
+			  "done",
+			  G_CALLBACK (done_printing_cb),
+			  tab);
+
+	if (is_preview)
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINT_PREVIEWING);
+	else
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINTING);
+
+	setup = get_page_setup (tab);
+	settings = get_print_settings (tab);
+
+	res = gedit_print_job_print (tab->priv->print_job,
+				     print_action,
+				     setup,
+				     settings,
+				     GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))),
+				     &error);
+
+	// TODO: manage res in the correct way
+	if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
 	{
 		/* FIXME: go in error state */
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-		g_warning ("Async print preview failed");
-		g_object_unref (pjob);
+		g_warning ("Async print preview failed (%s)", error->message);
+		g_object_unref (tab->priv->print_job);
+		g_error_free (error);
 	}
 }
 
+void 
+_gedit_tab_print (GeditTab     *tab)
+{
+	g_return_if_fail (GEDIT_IS_TAB (tab));
+
+	/* FIXME: currently we can have just one printoperation going on
+	 * at a given time, so before starting the print we close the preview.
+	 * Would be nice to handle it properly though */
+	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
+		gtk_widget_destroy (tab->priv->print_preview);
+	}
+
+	gedit_tab_print_or_print_preview (tab,
+					  GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG);
+}
+
+void
+_gedit_tab_print_preview (GeditTab     *tab)
+{
+	g_return_if_fail (GEDIT_IS_TAB (tab));
+
+	gedit_tab_print_or_print_preview (tab,
+					  GTK_PRINT_OPERATION_ACTION_PREVIEW);
+}
+
 void 
 _gedit_tab_mark_for_closing (GeditTab *tab)
 {
