@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-utils.c 5798 2007-08-14 22:30:35Z paolo $
+ * $Id: gedit-utils.c 6379 2008-08-10 16:40:21Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,15 +40,10 @@
 #include <sys/time.h>
 #include <fcntl.h>
 #include <string.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xatom.h>
 
-#include <gdk/gdkx.h>
 #include <glib/gunicode.h>
 #include <glib/gi18n.h>
-#include <glade/glade-xml.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
 
 #include "gedit-utils.h"
 
@@ -57,25 +52,27 @@
 #include "gedit-debug.h"
 #include "gedit-convert.h"
 
+/* For the workspace/viewport stuff */
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/Xatom.h>
+#endif
+
 #define STDIN_DELAY_MICROSECONDS 100000
 
 /* Returns true if uri is a file: uri and is not a chained uri */
 gboolean
 gedit_utils_uri_has_file_scheme (const gchar *uri)
 {
-	gchar *canonical_uri;
-	gchar *tmp;
+	GFile *gfile;
 	gboolean res;
 
-	canonical_uri = gnome_vfs_make_uri_canonical (uri);
-	g_return_val_if_fail (canonical_uri != NULL, FALSE);
-
-	tmp = gnome_vfs_get_local_path_from_uri (canonical_uri);
-	res = (tmp != NULL);
+	gfile = g_file_new_for_uri (uri);
+	res = g_file_has_uri_scheme (gfile, "file");
 	
-	g_free (canonical_uri);
-	g_free (tmp);
-
+	g_object_unref (gfile);
 	return res;
 }
 
@@ -83,18 +80,17 @@ gedit_utils_uri_has_file_scheme (const gchar *uri)
 gboolean
 gedit_utils_uri_has_writable_scheme (const gchar *uri)
 {
-	gchar *canonical_uri;
+	GFile *gfile;
 	gchar *scheme;
 	GSList *writable_schemes;
 	gboolean res;
 
-	canonical_uri = gnome_vfs_make_uri_canonical (uri);
-	g_return_val_if_fail (canonical_uri != NULL, FALSE);
+	gfile = g_file_new_for_uri (uri);
+	scheme = g_file_get_uri_scheme (gfile);
 
-	scheme = gnome_vfs_get_uri_scheme (canonical_uri);
 	g_return_val_if_fail (scheme != NULL, FALSE);
 
-	g_free (canonical_uri);
+	g_object_unref (gfile);
 
 	writable_schemes = gedit_prefs_manager_get_writable_vfs_schemes ();
 
@@ -339,19 +335,17 @@ gedit_utils_set_atk_relation (GtkWidget *obj1,
 gboolean
 gedit_utils_uri_exists (const gchar* text_uri)
 {
-	GnomeVFSURI *uri;
+	GFile *gfile;
 	gboolean res;
 		
 	g_return_val_if_fail (text_uri != NULL, FALSE);
 	
 	gedit_debug_message (DEBUG_UTILS, "text_uri: %s", text_uri);
 
-	uri = gnome_vfs_uri_new (text_uri);
-	g_return_val_if_fail (uri != NULL, FALSE);
+	gfile = g_file_new_for_uri (text_uri);
+	res = g_file_query_exists (gfile, NULL);
 
-	res = gnome_vfs_uri_exists (uri);
-
-	gnome_vfs_uri_unref (uri);
+	g_object_unref (gfile);
 
 	gedit_debug_message (DEBUG_UTILS, res ? "TRUE" : "FALSE");
 
@@ -373,9 +367,15 @@ gedit_utils_escape_search_text (const gchar* text)
 
     	length = strlen (text);
 
+	/* no escape when typing.
+	 * The short circuit works only for ascii, but we only
+	 * care about not escaping a single '\' */
+	if (length == 1)
+		return g_strdup (text);
+
 	str = g_string_new ("");
 
-  	p = text;
+	p = text;
   	end = text + length;
 
   	while (p != end)
@@ -394,6 +394,9 @@ gedit_utils_escape_search_text (const gchar* text)
 			case '\t':
           			g_string_append (str, "\\t");
           			break;
+			case '\\':
+          			g_string_append (str, "\\\\");
+          			break;
         		default:
           			g_string_append_len (str, p, next - p);
           			break;
@@ -487,7 +490,6 @@ gedit_utils_get_stdin (void)
 	GString * file_contents;
 	gchar *tmp_buf = NULL;
 	guint buffer_length;
-	GnomeVFSResult	res;
 	fd_set rfds;
 	struct timeval tv;
 	
@@ -514,8 +516,6 @@ gedit_utils_get_stdin (void)
 
 		if (ferror (stdin) != 0)
 		{
-			res = gnome_vfs_result_from_errno (); 
-		
 			g_free (tmp_buf);
 			g_string_free (file_contents, TRUE);
 			return NULL;
@@ -610,43 +610,6 @@ gedit_utils_escape_underscores (const gchar* text,
 	return g_string_free (str, FALSE);
 }
 
-/*
- * Replaces '/' with '-' to avoid problems in xml paths.
- */
-gchar *
-gedit_utils_escape_slashes (const gchar *text,
-			    gssize       length)
-{
-	GString *str;
-	const gchar *p;
-	const gchar *end;
-
-	g_return_val_if_fail (text != NULL, NULL);
-
-	if (length < 0)
-		length = strlen (text);
-
-	str = g_string_sized_new (length);
-
-	p = text;
-	end = text + length;
-
-	while (p != end)
-	{
-		const gchar *next;
-		next = g_utf8_next_char (p);
-
-		if (*p == '/')
-			g_string_append (str, "-");
-		else
-			g_string_append_len (str, p, next - p);
-
-		p = next;
-	}
-
-	return g_string_free (str, FALSE);
-}
-
 /* the following functions are taken from eel */
 
 gchar *
@@ -817,6 +780,7 @@ gedit_utils_replace_home_dir_with_tilde (const gchar *uri)
 guint
 gedit_utils_get_current_workspace (GdkScreen *screen)
 {
+#ifdef GDK_WINDOWING_X11
 	GdkWindow *root_win;
 	GdkDisplay *display;
 	Atom type;
@@ -824,6 +788,7 @@ gedit_utils_get_current_workspace (GdkScreen *screen)
 	gulong nitems;
 	gulong bytes_after;
 	guint *current_desktop;
+	gint err, result;
 	guint ret = 0;
 
 	g_return_val_if_fail (GDK_IS_SCREEN (screen), 0);
@@ -831,19 +796,26 @@ gedit_utils_get_current_workspace (GdkScreen *screen)
 	root_win = gdk_screen_get_root_window (screen);
 	display = gdk_screen_get_display (screen);
 
-	XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
-			    gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
-			    0, G_MAXLONG,
-			    False, XA_CARDINAL, &type, &format, &nitems,
-			    &bytes_after, (gpointer)&current_desktop);
+	gdk_error_trap_push ();
+	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
+				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_CURRENT_DESKTOP"),
+				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
+				     &bytes_after, (gpointer) &current_desktop);
+	err = gdk_error_trap_pop ();
+
+	if (err != Success || result != Success)
+		return ret;
 
 	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-	{
 		ret = current_desktop[0];
-		XFree (current_desktop);
-	}
 
+	XFree (current_desktop);
 	return ret;
+#else
+	/* FIXME: on mac etc proably there are native APIs
+	 * to get the current workspace etc */
+	return 0;
+#endif
 }
 
 /**
@@ -856,6 +828,7 @@ gedit_utils_get_current_workspace (GdkScreen *screen)
 guint
 gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 {
+#ifdef GDK_WINDOWING_X11
 	GdkWindow *window;
 	GdkDisplay *display;
 	Atom type;
@@ -863,6 +836,7 @@ gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 	gulong nitems;
 	gulong bytes_after;
 	guint *workspace;
+	gint err, result;
 	guint ret = GEDIT_ALL_WORKSPACES;
 
 	g_return_val_if_fail (GTK_IS_WINDOW (gtkwindow), 0);
@@ -871,19 +845,85 @@ gedit_utils_get_window_workspace (GtkWindow *gtkwindow)
 	window = GTK_WIDGET (gtkwindow)->window;
 	display = gdk_drawable_get_display (window);
 
-	XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
-			    gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
-			    0, G_MAXLONG,
-			    False, XA_CARDINAL, &type, &format, &nitems,
-			    &bytes_after, (gpointer)&workspace);
+	gdk_error_trap_push ();
+	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (window),
+				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_DESKTOP"),
+				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
+				     &bytes_after, (gpointer) &workspace);
+	err = gdk_error_trap_pop ();
+
+	if (err != Success || result != Success)
+		return ret;
 
 	if (type == XA_CARDINAL && format == 32 && nitems > 0)
-	{
 		ret = workspace[0];
-		XFree (workspace);
+
+	XFree (workspace);
+	return ret;
+#else
+	/* FIXME: on mac etc proably there are native APIs
+	 * to get the current workspace etc */
+	return 0;
+#endif
+}
+
+/**
+ * gedit_utils_get_current_viewport: Get the current viewport origin
+ *
+ * Get the currently visible viewport origin for the #GdkScreen.
+ *
+ * If the X11 window property isn't found, (0, 0) is returned.
+ */
+void
+gedit_utils_get_current_viewport (GdkScreen    *screen,
+				  gint         *x,
+				  gint         *y)
+{
+#ifdef GDK_WINDOWING_X11
+	GdkWindow *root_win;
+	GdkDisplay *display;
+	Atom type;
+	gint format;
+	gulong nitems;
+	gulong bytes_after;
+	gulong *coordinates;
+	gint err, result;
+
+	g_return_if_fail (GDK_IS_SCREEN (screen));
+	g_return_if_fail (x != NULL && y != NULL);
+
+	/* Default values for the viewport origin */
+	*x = 0;
+	*y = 0;
+
+	root_win = gdk_screen_get_root_window (screen);
+	display = gdk_screen_get_display (screen);
+
+	gdk_error_trap_push ();
+	result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display), GDK_WINDOW_XID (root_win),
+				     gdk_x11_get_xatom_by_name_for_display (display, "_NET_DESKTOP_VIEWPORT"),
+				     0, G_MAXLONG, False, XA_CARDINAL, &type, &format, &nitems,
+				     &bytes_after, (void*) &coordinates);
+	err = gdk_error_trap_pop ();
+
+	if (err != Success || result != Success)
+		return;
+
+	if (type != XA_CARDINAL || format != 32 || nitems < 2)
+	{
+		XFree (coordinates);
+		return;
 	}
 
-	return ret;
+	*x = coordinates[0];
+	*y = coordinates[1];
+	XFree (coordinates);
+#else
+	/* FIXME: on mac etc proably there are native APIs
+	 * to get the current workspace etc */
+	*x = 0;
+	*y = 0;
+#endif
 }
 
 void
@@ -891,7 +931,7 @@ gedit_utils_activate_url (GtkAboutDialog *about,
 			  const gchar    *url,
 			  gpointer        data)
 {
-	gnome_vfs_url_show (url);
+	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (about)), url, GDK_CURRENT_TIME, NULL);
 }
 
 static gboolean
@@ -951,110 +991,132 @@ gedit_utils_is_valid_uri (const gchar *uri)
 	return TRUE;
 }
 
+static GtkWidget *
+handle_builder_error (const gchar *message,
+		      ...)
+{
+	GtkWidget *label;
+	gchar *msg;
+	gchar *msg_plain;
+	va_list args;
+
+	va_start (args, message);
+	va_arg (args, const gchar *);
+	msg_plain = g_strdup_vprintf (message, args);
+
+	msg = g_strconcat ("<span size=\"large\" weight=\"bold\">",
+			msg_plain, "</span>\n\n",
+			_("Please check your installation."), NULL);
+	label = gtk_label_new (msg);
+
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+
+	g_free (msg_plain);
+	g_free (msg);
+
+	gtk_misc_set_padding (GTK_MISC (label), 5, 5);
+
+	return label;
+}
 
 /**
- * gedit_utils_get_glade_widgets:
- * @filename: the path to the glade file
- * @root_node: the root node in the glade file
+ * gedit_utils_get_ui_objects:
+ * @filename: the path to the gtk builder file
+ * @root_objects: a NULL terminated list of root objects to load or NULL to
+ *                load all objects
  * @error_widget: a pointer were a #GtkLabel
- * @widget_name: the name of the first widget
- * @...: a pointer were the first widget is returned, followed by more
- *       name / widget pairs and terminated by NULL.
+ * @object_name: the name of the first object
+ * @...: a pointer were the first object is returned, followed by more
+ *       name / object pairs and terminated by NULL.
  *
- * This function gets the requested widgets from a glade file. In case
+ * This function gets the requested objects from a GtkBuilder ui file. In case
  * of error it returns FALSE and sets error_widget to a GtkLabel containing
  * the error message to display.
  *
  * Returns FALSE if an error occurs, TRUE on success.
  */
 gboolean
-gedit_utils_get_glade_widgets (const gchar *filename,
-			       const gchar *root_node,
-			       GtkWidget **error_widget,
-			       const gchar *widget_name,
-			       ...)
+gedit_utils_get_ui_objects (const gchar  *filename,
+			    gchar       **root_objects,
+			    GtkWidget   **error_widget,
+			    const gchar  *object_name,
+			    ...)
 {
-	GtkWidget *label;
-	GladeXML *gui;
+
+	GtkBuilder *builder;
 	va_list args;
 	const gchar *name;
-	gchar *msg;
+	GError *error = NULL;
 	gchar *filename_markup;
-	gchar *msg_plain;
 	gboolean ret = TRUE;
 
 	g_return_val_if_fail (filename != NULL, FALSE);
 	g_return_val_if_fail (error_widget != NULL, FALSE);
-	g_return_val_if_fail (widget_name != NULL, FALSE);
+	g_return_val_if_fail (object_name != NULL, FALSE);
 
+	filename_markup = g_markup_printf_escaped ("<i>%s</i>", filename);
 	*error_widget = NULL;
 
-	gui = glade_xml_new (filename, root_node, NULL);
-	if (!gui)
+	builder = gtk_builder_new ();
+	
+	if (root_objects != NULL)
+		gtk_builder_add_objects_from_file (builder, 
+						   filename, 
+						   root_objects, 
+						   &error);
+	else
+		gtk_builder_add_from_file (builder,
+					   filename,
+					   &error);
+
+	if (error != NULL)
 	{
-		filename_markup = g_markup_printf_escaped ("<i>%s</i>", filename);
-		msg_plain = g_strdup_printf (_("Unable to find file %s."),
-				filename_markup);
-		msg = g_strconcat ("<span size=\"large\" weight=\"bold\">",
-				msg_plain, "</span>\n\n",
-				_("Please check your installation."), NULL);
-		label = gtk_label_new (msg);
-
-		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-		
+		*error_widget = handle_builder_error (_("Unable to open ui file %s. Error: %s"),
+						      filename_markup,
+						      error->message);
+		g_error_free (error);
 		g_free (filename_markup);
-		g_free (msg_plain);
-		g_free (msg);
-
-		gtk_misc_set_padding (GTK_MISC (label), 5, 5);
- 		
-		*error_widget = label;
 
 		return FALSE;
 	}
 
-	va_start (args, widget_name);
-	for (name = widget_name; name; name = va_arg (args, const gchar *) )
+	va_start (args, object_name);
+	for (name = object_name; name; name = va_arg (args, const gchar *) )
 	{
-		GtkWidget **wid;
+		GObject **gobj;
 
-		wid = va_arg (args, GtkWidget **);
-		*wid = glade_xml_get_widget (gui, name);
-		if (*wid == NULL)
+		gobj = va_arg (args, GObject **);
+		*gobj = gtk_builder_get_object (builder, name);
+
+		if (!*gobj)
 		{
-			g_warning ("Cannot find widget '%s' inside file '%s'.",
-				   name,
-				   filename);
-				   
-			filename_markup = g_markup_printf_escaped ("<i>%s</i>", filename);
-			msg_plain = g_strdup_printf (
-					_("Unable to find the required widgets inside file %s."),
-					filename_markup);
-			msg = g_strconcat ("<span size=\"large\" weight=\"bold\">",
-					msg_plain, "</span>\n\n",
-					_("Please check your installation."), NULL);
-			label = gtk_label_new (msg);
-
-			gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-			gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-			
-			g_free (filename_markup);
-			g_free (msg_plain);
-			g_free (msg);
-
-			gtk_misc_set_padding (GTK_MISC (label), 5, 5);
- 			
-			*error_widget = label;
-
+			*error_widget = handle_builder_error (_("Unable to find the object '%s' inside file %s."), 
+							      name, 
+							      filename_markup),
 			ret = FALSE;
-
 			break;
 		}
+
+		/* we return a new ref for the root objects,
+		 * the others are already reffed by their parent root object */
+		if (root_objects != NULL)
+		{
+			gint i;
+
+			for (i = 0; root_objects[i] != NULL; ++i)
+			{
+				if ((strcmp (name, root_objects[i]) == 0))
+				{
+					g_object_ref (*gobj);
+				}
+			}
+		}
 	}
 	va_end (args);
 
-	g_object_unref (gui);
+	g_free (filename_markup);
+	g_object_unref (builder);
 
 	return ret;
 }
@@ -1062,13 +1124,14 @@ gedit_utils_get_glade_widgets (const gchar *filename,
 gchar *
 gedit_utils_make_canonical_uri_from_shell_arg (const gchar *str)
 {	
+	GFile *gfile;
 	gchar *uri;
-	gchar *canonical_uri;
 
 	g_return_val_if_fail (str != NULL, NULL);
 	g_return_val_if_fail (*str != '\0', NULL);
 	
 	/* Note for the future: 
+	 * FIXME: is still still relevant?
 	 *
 	 * <federico> paolo: and flame whoever tells 
 	 * you that file:///gnome/test_files/hëllò 
@@ -1083,21 +1146,119 @@ gedit_utils_make_canonical_uri_from_shell_arg (const gchar *str)
 	 * <paolo>: I will use gedit_utils_is_valid_uri ()
 	 *
 	 */
-	 
-	uri = gnome_vfs_make_uri_from_shell_arg (str);
-	canonical_uri = gnome_vfs_make_uri_canonical (uri);
+
+	gfile = g_file_new_for_commandline_arg (str);
+	uri = g_file_get_uri (gfile);
+	g_object_unref (gfile);
+
+	if (gedit_utils_is_valid_uri (uri))
+		return uri;
+	
 	g_free (uri);
-	
-	/* g_print ("URI: %s\n", canonical_uri); */
-	
-	if (gedit_utils_is_valid_uri (canonical_uri))
-		return canonical_uri;
-	
 	return NULL;
 }
 
 /**
- * gedit_utils_format_uri_for_display:
+ * gedit_utils_file_has_parent:
+ * @gfile: the GFile to check the parent for
+ *
+ * Return TRUE if the specified gfile has a parent (is not the root), FALSE
+ * otherwise
+ */
+gboolean
+gedit_utils_file_has_parent (GFile *gfile)
+{
+	GFile *parent;
+	gboolean ret;
+	
+	parent = g_file_get_parent (gfile);
+	ret = parent != NULL;
+	
+	if (parent)
+		g_object_unref (parent);
+	
+	return ret;
+}
+
+/**
+ * gedit_utils_basename_for_display:
+ * @uri: uri for which the basename should be displayed
+ *
+ * Return the basename of a file suitable for display to users.
+ */
+gchar *
+gedit_utils_basename_for_display (gchar const *uri)
+{
+	gchar *name;
+	GFile *gfile;
+	gchar *hn;
+	
+	g_return_val_if_fail (uri != NULL, NULL);
+	
+	gfile = g_file_new_for_uri (uri);
+	
+	/* First, try to query the display name, but only on local files */
+	if (g_file_has_uri_scheme (gfile, "file"))
+	{
+		GFileInfo *info;
+		info = g_file_query_info (gfile,
+					  G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, 
+					  G_FILE_QUERY_INFO_NONE, 
+					  NULL, 
+					  NULL);
+
+		if (info)
+		{
+			/* Simply get the display name to use as the basename */
+			name = g_strdup (g_file_info_get_display_name (info));
+			g_object_unref (info);
+		}
+		else
+		{
+			/* This is a local file, and therefore we will use
+			 * g_filename_display_basename on the local path */
+			gchar *local_path;
+		
+			local_path = g_file_get_path (gfile);
+			name = g_filename_display_basename (local_path);
+			g_free (local_path);
+		}
+	}
+	else if (gedit_utils_file_has_parent (gfile) || !gedit_utils_decode_uri (uri, NULL, NULL, &hn, NULL, NULL))
+	{
+		/* For remote files with a parent (so not just http://foo.com)
+		   or remote file for which the decoding of the host name fails,
+		   use the _parse_name and take basename of that */
+		gchar *parse_name;
+		gchar *base;
+
+		parse_name = g_file_get_parse_name (gfile);
+		base = g_filename_display_basename (parse_name);
+		name = g_uri_unescape_string (base, NULL);
+		
+		g_free (base);		
+		g_free (parse_name);
+	}
+	else
+	{
+		/* display '/ on <host>' using the decoded host */
+		gchar *hn_utf8;
+		hn_utf8 = gedit_utils_make_valid_utf8 (hn);
+		
+		/* Translators: '/ on <remote-share>' */
+		name = g_strdup_printf (_("/ on %s"), hn_utf8);
+		
+		g_free (hn_utf8);
+		g_free (hn);
+	}
+
+	g_object_unref (gfile);
+
+	return name;
+}
+
+/**
+ * gedit_utils_uri_for_display:
  * @uri: uri to be displayed.
  *
  * Filter, modify, unescape and change @uri to make it appropriate
@@ -1110,43 +1271,25 @@ gedit_utils_make_canonical_uri_from_shell_arg (const gchar *str)
  * <li>All other uri appear as expected.</li>
  * </ul>
  *
- * This function is very similar to gnome_vfs_format_uri_for_display but remove
- * the password from the resulting string
+ * This function is a convenient wrapper for g_file_get_parse_name
  *
  * Return value: a string which represents @uri and can be displayed.
  */
 gchar *
-gedit_utils_format_uri_for_display (const gchar *uri)
+gedit_utils_uri_for_display (const gchar *uri)
 {
-	GnomeVFSURI *vfs_uri;
+	GFile *gfile;
+	gchar *parse_name;
+	gchar *unescape;
 	
-	g_return_val_if_fail (uri != NULL, NULL);
-
-	/* Note: vfs_uri may be NULL for some valid but
-	 * unsupported uris */
-	vfs_uri = gnome_vfs_uri_new (uri);
+	gfile = g_file_new_for_uri (uri);
 	
-	if (vfs_uri == NULL)
-	{
-		/* We may disclose the password here, but there is nothing we
-		 * can do since we cannot get a valid vfs_uri */
-		return gnome_vfs_format_uri_for_display (uri);
-	}
-	else
-	{	
-		gchar *name;
-		gchar *uri_for_display;
-		
-		name = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_PASSWORD);
-		g_return_val_if_fail (name != NULL, gnome_vfs_format_uri_for_display (uri));
-		
-		uri_for_display = gnome_vfs_format_uri_for_display (name);
-		g_free (name);
-		
-		gnome_vfs_uri_unref (vfs_uri);
-		
-		return uri_for_display;
-	}
+	parse_name = g_file_get_parse_name (gfile);
+	unescape = g_uri_unescape_string (parse_name, NULL);
+	g_free (parse_name);
+	
+	g_object_unref (gfile);
+	return unescape;
 }
 
 /**
@@ -1190,3 +1333,154 @@ gedit_utils_drop_get_uris (GtkSelectionData *selection_data)
 
 	return uri_list;
 }
+
+static void
+null_ptr (gchar **ptr)
+{
+	if (ptr)
+		*ptr = NULL;
+}
+
+/**
+ * gedit_utils_decode_uri:
+ * @uri: the uri to decode
+ * @scheme: return value pointer for the uri's scheme (e.g. http, sftp, ...)
+ * @user: return value pointer for the uri user info
+ * @port: return value pointer for the uri port
+ * @host: return value pointer for the uri host
+ * @path: return value pointer for the uri path
+ *
+ * Parse and break an uri apart in its individual components like the uri
+ * scheme, user info, port, host and path. The return value pointer can be
+ * NULL to ignore certain parts of the uri. If the function returns TRUE, then
+ * all return value pointers should be freed using g_free
+ * 
+ * Return value: TRUE if the uri could be properly decoded, FALSE otherwise.
+ */
+gboolean
+gedit_utils_decode_uri (const gchar *uri,
+			gchar **scheme,
+			gchar **user,
+			gchar **host,
+			gchar **port,
+			gchar **path
+)
+{
+	/* Largely copied from glib/gio/gdummyfile.c:_g_decode_uri. This 
+	 * functionality should be in glib/gio, but for now we implement it
+	 * ourselves (see bug #546182) */
+
+	const char *p, *in, *hier_part_start, *hier_part_end;
+	char *out;
+	char c;
+
+	/* From RFC 3986 Decodes:
+	 * URI = scheme ":" hier-part [ "?" query ] [ "#" fragment ]
+	 */ 
+
+	p = uri;
+	
+	null_ptr (scheme);
+	null_ptr (user);
+	null_ptr (port);
+	null_ptr (host);
+	null_ptr (path);
+
+	/* Decode scheme:
+	 * scheme = ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
+	 */
+
+	if (!g_ascii_isalpha (*p))
+		return FALSE;
+
+	while (1)
+	{
+		c = *p++;
+
+		if (c == ':')
+			break;
+
+		if (!(g_ascii_isalnum(c) ||
+		      c == '+' ||
+		      c == '-' ||
+		      c == '.'))
+			return FALSE;
+	}
+	
+	if (scheme)
+	{
+		*scheme = g_malloc (p - uri);
+		out = *scheme;
+	
+		for (in = uri; in < p - 1; in++)
+			*out++ = g_ascii_tolower (*in);
+			
+		*out = '\0';
+	}
+	
+	hier_part_start = p;
+	hier_part_end = p + strlen (p);
+	
+	if (hier_part_start[0] == '/' && hier_part_start[1] == '/')
+	{
+		const char *authority_start, *authority_end;
+		const char *userinfo_start, *userinfo_end;
+		const char *host_start, *host_end;
+		const char *port_start;
+		
+		authority_start = hier_part_start + 2;
+		/* authority is always followed by / or nothing */
+		authority_end = memchr (authority_start, '/', hier_part_end - authority_start);
+		
+		if (authority_end == NULL)
+			authority_end = hier_part_end;
+
+		/* 3.2:
+		 * authority = [ userinfo "@" ] host [ ":" port ]
+		 */
+
+		userinfo_end = memchr (authority_start, '@', authority_end - authority_start);
+		
+		if (userinfo_end)
+		{
+			userinfo_start = authority_start;
+			
+			if (user)
+				*user = g_uri_unescape_segment (userinfo_start, userinfo_end, NULL);
+			
+			if (user && *user == NULL)
+			{
+				if (scheme)
+					g_free (*scheme);
+
+				return FALSE;
+			}
+	
+			host_start = userinfo_end + 1;
+		}
+		else
+			host_start = authority_start;
+
+		port_start = memchr (host_start, ':', authority_end - host_start);
+
+		if (port_start)
+		{
+			host_end = port_start++;
+
+			if (port)
+				*port = g_strndup (port_start, authority_end - port_start);
+		}
+		else
+			host_end = authority_end;
+
+		if (host)
+			*host = g_strndup (host_start, host_end - host_start);
+
+		hier_part_start = authority_end;
+	}
+
+	if (path)
+		*path = g_uri_unescape_segment (hier_part_start, hier_part_end, "/");
+	
+	return TRUE;
+}
