@@ -71,7 +71,7 @@ dialog_response_handler (GtkDialog *dlg,
 	if (res_id == GTK_RESPONSE_HELP)
 	{
 		gedit_help_display (GTK_WINDOW (dlg),
-				    "gedit.xml",
+				    NULL,
 				    "gedit-spell-checker-plugin");
 
 		g_signal_stop_emission_by_name (dlg, "response");
@@ -122,12 +122,16 @@ gedit_spell_language_dialog_init (GeditSpellLanguageDialog *dlg)
 	gboolean ret;
 	GtkCellRenderer *cell;
 	GtkTreeViewColumn *column;
-
+	gchar *root_objects[] = {
+		"content",
+		NULL
+	};
+	
 	gtk_dialog_add_buttons (GTK_DIALOG (dlg),
-				GTK_STOCK_OK,
-				GTK_RESPONSE_OK,
 				GTK_STOCK_CANCEL,
 				GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OK,
+				GTK_RESPONSE_OK,
 				GTK_STOCK_HELP,
 				GTK_RESPONSE_HELP,
 				NULL);
@@ -148,12 +152,12 @@ gedit_spell_language_dialog_init (GeditSpellLanguageDialog *dlg)
 			  G_CALLBACK (dialog_response_handler),
 			  NULL);
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "languages-dialog.glade2",
-					     "content",
-					     &error_widget,
-					     "content", &content,
-					     "languages_treeview", &dlg->languages_treeview,
-					     NULL);
+	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "languages-dialog.ui",
+					  root_objects,	
+					  &error_widget,
+					  "content", &content,
+					  "languages_treeview", &dlg->languages_treeview,
+					  NULL);
 
 	if (!ret)
 	{
@@ -167,7 +171,7 @@ gedit_spell_language_dialog_init (GeditSpellLanguageDialog *dlg)
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 			    content, TRUE, TRUE, 0);
-
+	g_object_unref (content);
 	gtk_container_set_border_width (GTK_CONTAINER (content), 5);
 
 	dlg->model = GTK_TREE_MODEL (gtk_list_store_new (ENCODING_NUM_COLS,
